import axios from "axios";
import Router from "@/router";
import { Notification } from "@arco-design/web-vue";
import { useSystemStore } from "@/store/modules/systemStore";
import moment from 'moment-timezone';
import { fn_logout } from "@/utils";

// 用于存储请求和对应的 AbortController
const requestMap = new Map();
const requestCountMap = new Map(); // 用于记录请求的次数

const axiosInstance = axios.create({
  baseURL: import.meta.env.BASE_URL + "api", // 设置 API 基础 URL
  timeout: 300000, // 设置请求超时时间
});

const requestState = {
  success: [200],
  beOverdue: [886],
  NotAccessRight: [500],
  exception: [400],
};

const pathArr = ["/api/admin/system/login", "/api/logout"];

axiosInstance.interceptors.request.use(
  (config) => {
    const systemStore = useSystemStore();
    systemStore.localLoading(true);

    if (!pathArr.includes(config.url)) {
      const token = localStorage.getItem("token");
      if (token && config.headers) {
        config.headers["Authorization"] = token;
      }
    }

    const requestKey = `${config.url}?${JSON.stringify(config.params)}`;

    // 判断同一个接口请求次数
    const currentCount = requestCountMap.get(requestKey) || 0;
    if (currentCount >= 5) {
      // 如果请求次数大于等于5次，取消当前请求
      const controller = requestMap.get(requestKey);
      controller?.abort();
      console.log(`请求 ${requestKey} 已被取消，超过请求次数限制`);
      Notification.warning({
        title: "请求频繁",
        content: `请求 ${config.url} 过于频繁，已取消当前请求。`,
      });
      return Promise.reject("请求过于频繁，已取消");
    }

    // 更新请求次数
    requestCountMap.set(requestKey, currentCount + 1);

    // 创建新的 AbortController
    const controller = new AbortController();
    config.signal = controller.signal;

    // 存储当前请求的 AbortController
    requestMap.set(requestKey, controller);

    return config;
  },
  (err) => {
    const systemStore = useSystemStore();
    systemStore.localLoading();
    return Promise.reject(err);
  }
);

// 响应拦截器
axiosInstance.interceptors.response.use(
  (res) => {
    const systemStore = useSystemStore();
    systemStore.localLoading();
    const { code, data, message: msg } = res.data;

    // 获取当前请求的 key
    const requestKey = `${res.config.url}?${JSON.stringify(res.config.params)}`;

    // 请求完成后移除 AbortController
    if (requestMap.has(requestKey)) {
      requestMap.delete(requestKey);
    }

    // 清理请求次数
    if (requestCountMap.has(requestKey)) {
      requestCountMap.delete(requestKey);
    }

    if (code === 200) {
      const dataList = {
        code: code,
        message: res.data.msg,
        data: DataViewsProvider(data),
      };
      return Promise.resolve(dataList);
    }

    if (requestState.NotAccessRight.includes(code)) {
      Notification.warning({
        title: "系统信息",
        content: msg,
      });
      return Promise.reject(msg);
    }

    if (requestState.exception.includes(code)) {
      Notification.warning({
        title: "系统提示",
        content: msg,
      });
      return Promise.reject(msg);
    }

    return Promise.resolve(data);
  },
  (err) => {
    const systemStore = useSystemStore();
    systemStore.localLoading();

    const requestKey = `${err.config.url}?${JSON.stringify(err.config.params)}`;

    // 请求完成后移除 AbortController
    if (requestMap.has(requestKey)) {
      requestMap.delete(requestKey);
    }

    // 清理请求次数
    if (requestCountMap.has(requestKey)) {
      requestCountMap.delete(requestKey);
    }

    const msg = err.response?.data ? err.response.data.message : "";
    Notification.warning({
      title: "系统信息",
      content: msg,
    });

    if (requestState.beOverdue.includes(err.status)) {
      fn_logout(Router);
      Router.push("/");
      return undefined;
    }

    return Promise.reject(err);
  }
);

function timeLoadTemeYear(value) {
  if (!value) return;
  const systemStore = useSystemStore();
  const timezone = systemStore.getUserIp?.timezone || JSON.parse(localStorage.getItem('IPCONFIG'))?.timezone;
  if (timezone) {
    let localDate = moment.utc(value).tz(timezone);

    if (!localDate.isValid()) {
      return null;
    }

    const localTime = localDate.format('YYYY-MM-DD HH:mm:ss');
    return localTime;
  }

  return value;
}

function DataViewsProvider(data) {
  if (!data) return data;

  if (data.records && Array.isArray(data.records)) {
    return {
      ...data,
      records: data.records.map(item => {
        const transformedItem = { ...item };

        if (item.createdAt) transformedItem.createdAt = timeLoadTemeYear(item.createdAt);
        if (item.updatedAt) transformedItem.updatedAt = timeLoadTemeYear(item.updatedAt);
        if (item.endDate) transformedItem.endDate = timeLoadTemeYear(item.endDate);
        if (item.activeTime) transformedItem.activeTime = timeLoadTemeYear(item.activeTime);

        return transformedItem;
      })
    };
  }

  if (Array.isArray(data)) {
    return data.map(item => {
      const transformedItem = { ...item };

      if (item.createdAt) transformedItem.createdAt = timeLoadTemeYear(item.createdAt);
      if (item.updatedAt) transformedItem.updatedAt = timeLoadTemeYear(item.updatedAt);
      if (item.endDate) transformedItem.endDate = timeLoadTemeYear(item.endDate);
      if (item.activeTime) transformedItem.activeTime = timeLoadTemeYear(item.activeTime);

      return transformedItem;
    });
  }

  return data;
}

export default axiosInstance;
